<?php

namespace App\Controller\Config;

use App\Entity\Recipe;
use App\Form\Config\RecipeEditForm;
use App\Misc\FlashType;
use App\Repository\RecipeRepository;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Security\Http\Attribute\IsGranted;

/**
 * Controller for the configuration pages of material
 */
#[Route('/Config/Recipe')]
#[IsGranted('IS_AUTHENTICATED')]
class RecipeController extends AbstractController {
    /**
     * @var EntityManagerInterface The entity manager
     */
    private readonly EntityManagerInterface $entityManager;
    /**
     * @var RecipeRepository The recipe repository
     */
    private readonly RecipeRepository $recipeRepository;

    /**
     * Initialization
     *
     * @param EntityManagerInterface $entityManager    The entity manager
     * @param RecipeRepository       $recipeRepository The material repository
     */
    public function __construct (EntityManagerInterface $entityManager, RecipeRepository $recipeRepository) {
        $this->entityManager = $entityManager;
        $this->recipeRepository = $recipeRepository;
    }

    /**
     * List of materials
     *
     * @return Response The response
     */
    #[Route('/', name: 'config_recipe_list', alias: 'config_recipe')]
    public function list (): Response {
        return $this->render(
            'Config/Recipe/List.html.twig',
            [
                'recipes' => $this->recipeRepository->findAll(),
            ]
        );
    }

    /**
     * Edit/Create a recipe
     *
     * @param Request     $request The request
     * @param Recipe|null $recipe  The recipe to edit
     *
     * @return Response The response
     */
    #[Route('/Create', name: 'config_recipe_create')]
    #[Route('/Edit-{id}', name: 'config_recipe_edit')]
    public function edit (Request $request, ?Recipe $recipe = null): Response {
        $recipe ??= new Recipe();

        $form = $this->createForm(RecipeEditForm::class, $recipe);
        $form->handleRequest($request);
        if ($form->isSubmitted() && $form->isValid()) {
            $this->entityManager->persist($recipe);
            $this->entityManager->flush();

            $this->addFlash(FlashType::SUCCESS, 'La recette a bien été enregistré.');
            return $this->redirectToRoute('config_recipe_list');
        }

        return $this->render('Config/Recipe/Edit.html.twig', [
            'recipe' => $recipe,
            'form'   => $form->createView(),
        ]);
    }

    /**
     * Delete a recipe
     *
     * @param Recipe $recipe The recipe to delete
     *
     * @return Response The response
     */
    #[Route('/Delete-{id}', name: 'config_recipe_delete')]
    public function delete (Recipe $recipe): Response {
        $this->entityManager->remove($recipe);
        $this->entityManager->flush();

        $this->addFlash(FlashType::SUCCESS, 'La recette a bien été supprimée.');
        return $this->redirectToRoute('config_recipe_list');
    }
}
